use std::str::FromStr;

use clap::Parser;
use libvanity::vanity::VanityRequest;

#[derive(clap::Parser, Debug)]
#[clap(
    about = "ask some worker to generate a private key where public key matches some regular expression"
)]
struct Args {
    #[clap(short = 'c', long = "connect", help = "server address to connect to")]
    endpoint: String,

    #[clap(help = "pattern of public key")]
    pattern: String,

    #[clap(short, long, default_value = "WARN")]
    log_level: String,
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    let args = Args::parse();

    let level = tracing::Level::from_str(&args.log_level)?;
    tracing_subscriber::FmtSubscriber::builder()
        .with_max_level(level)
        .init();

    let mut client =
        libvanity::vanity::vanity_service_client::VanityServiceClient::connect(args.endpoint)
            .await?;

    let mut request = VanityRequest::default();
    request.pattern = args.pattern;
    let response = client.request_vanity(request).await?.into_inner();
    println!("{}", response.result);

    Ok(())
}
